# -*- coding: utf8 -*-
from __future__ import unicode_literals

import time

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment
from driverng.astm import HprimDate as HL7Date
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment


class SegmentAutoplak(Segment):
    rang_segment = HL7String((2, 1), obligatoire=True, max_length=4)


class AutoplakSegmentMSH(HeaderSegment):
    sending_application = HL7String((3, 1))
    sending_facility = HL7String((4, 1))
    receiving_application = HL7String((5, 1))
    receiving_facility = HL7String((6, 1))
    date = HL7Date(7)
    message_type = HL7String((9, 1), default="RSP")
    message_code = HL7String((9, 2), default="K11")
    message_label = HL7String((9, 3), default="RSP_K11")
    control_id = HL7String(10)
    processing_id = HL7String(11, default="P")
    version = HL7String(12, default="2.5")

    acknowledgment = HL7Foreign("MSA", "header")
    specimen = HL7Foreign("SPM", "header")
    specimen_container = HL7Foreign("SAC", "header")
    queries = HL7Foreign("QPD", "header")


class AutoplakSegmentMSA(SegmentAutoplak):
    acknowlegment_code = HL7String(2)
    message_control_id = HL7String(3)

    query_tag = HL7Foreign("QAK", "acknowledgment")

    def to_string(self, separators=None):
        self.rang_segment = "AA"
        return super(AutoplakSegmentMSA, self).to_string(separators=separators)


class AutoplakSegmentQAK(SegmentAutoplak):
    response_status = HL7String(3, default="OK")

    query_message = HL7Foreign("QPD", "query_tag")

    def to_string(self, separators=None):
        self.rang_segment = "MSG00001"
        return super(AutoplakSegmentQAK, self).to_string(separators=separators)


class AutoplakSegmentSPM(SegmentAutoplak):
    # Id du LIS
    specimen_id = HL7String((3, 1))
    # Id donné par l'Autoplak
    autoplak_id = HL7String((4, 1))
    specimen_type_code = HL7String((5, 1))
    specimen_type_text = HL7String((5, 2))
    specimen_role = HL7String(12, default="P")
    collection_date = HL7Date(18)

    specimen_container = HL7Foreign("SAC", "specimen")
    infos = HL7Foreign("ORC", "specimen")
    patient = HL7Foreign("PID", "specimen")


class AutoplakSegmentPID(SegmentAutoplak):
    first_name = HL7String((6, 1))
    last_name = HL7String((6, 2))
    birthdate = HL7Date(8, fmt_date="%Y%m%d")
    sex = HL7String(9)

    specimen_container = HL7Foreign("SAC", "patient")
    infos = HL7Foreign("ORC", "patient")


class AutoplakSegmentSAC(SegmentAutoplak):
    sample_identification = HL7String((4, 1))
    sample_name = HL7String((4, 2), default="AUTOPLAK")
    container_status = HL7String((9, 1), default="R")
    container_text = HL7String((9, 2), default="COMPLETED")
    # Identifies the type of the carrier
    carrier_id = HL7String((10, 1))
    # Identifies the position of the container in the carrier
    carrier_position = HL7String((12, 1))
    # Identifies the type of the tray
    tray_id = HL7String((13, 1))
    tray_identifier = HL7String((14, 1))
    # Identifies the position of the carrier in the tray
    tray_position = HL7String((15, 1))
    identifier = HL7String((31, 1))

    orders = HL7Foreign("OBR", "specimen")
    infos = HL7Foreign("ORC", "specimen")


class AutoplakSegmentOBR(SegmentAutoplak):
    placer_order_number = HL7String((3, 1))
    code = HL7String((5, 1))
    ordering_provider = HL7String(16)

    infos = HL7Foreign("ORC", "orders")
    results = HL7Foreign("OBX", "orders")


class AutoplakSegmentTQ1(SegmentAutoplak):
    priority = HL7String((10, 1))

    orders = HL7Foreign("OBR", "infos")


class AutoplakSegmentOBX(SegmentAutoplak):
    value_type = HL7String(3)
    observation_identifier = HL7String((4, 1))
    observation_text = HL7String((4, 2))
    value = HL7String(6)
    result_status = HL7String(12)

    infos = HL7Foreign("ORC", "results")


class AutoplakSegmentORC(SegmentAutoplak):
    order_control = HL7String(2)
    placer_order_number = HL7String(3)
    placer_identifier = HL7String((5, 1))
    date = HL7Date(10)

    sample_priority = HL7Foreign("TQ1", "infos")
    results = HL7Foreign("OBX", "infos")

    def to_string(self, separators=None):
        self.rang_segment = "NW"
        return super(AutoplakSegmentORC, self).to_string(separators=separators)


class AutoplakSegmentQPD(SegmentAutoplak):
    query_text = HL7String((2, 2), default="Work Order Step")
    query_tag = HL7String(3)
    specimen_identification = HL7String(4)

    specimen = HL7Foreign("SPM", "queries")
    queries_infos = HL7Foreign("RCP", "queries")

    def to_string(self, separators=None):
        self.rang_segment = "WOS"
        return super(AutoplakSegmentQPD, self).to_string(separators=separators)


class AutoplakSegmentRCP(SegmentAutoplak):
    query_priority = HL7String(1)
    response_modality = HL7String(3)


class Driver(AstmMessageV2):
    SPEC = {"code": "autoplak", "label": "Autoplak HL7"}

    SEGMENT_CLASS_MAP = {
        "MSH": AutoplakSegmentMSH,
        "SPM": AutoplakSegmentSPM,
        "PID": AutoplakSegmentPID,
        "SAC": AutoplakSegmentSAC,
        "OBR": AutoplakSegmentOBR,
        "TQ1": AutoplakSegmentTQ1,
        "OBX": AutoplakSegmentOBX,
        "ORC": AutoplakSegmentORC,
        "MSA": AutoplakSegmentMSA,
        "QAK": AutoplakSegmentQAK,
        "QPD": AutoplakSegmentQPD,
        "RCP": AutoplakSegmentRCP,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"

    OPTIONS = [
        AstmMessageV2.Option(
            "autoplak_sender_code", label="Code émetteur", default="LIS01"
        ),
        AstmMessageV2.Option(
            "autoplak_sender_label", label="Libellé émetteur", default="Microbiology"
        ),
        AstmMessageV2.Option(
            "autoplak_receiver_code", label="Code récepteur", default="AUTOPLAK01"
        ),
        AstmMessageV2.Option(
            "autoplak_receiver_label", label="Libellé récepteur", default="MicroLab"
        ),
    ]

    def __init__(self, **kwargs):
        self.autoplak_sender_code = kwargs.pop("autoplak_sender_code", "LIS01")
        self.autoplak_sender_label = kwargs.pop("autoplak_sender_label", "Microbiology")
        self.autoplak_receiver_code = kwargs.pop("autoplak_receiver_code", "AUTOPLAK01")
        self.autoplak_receiver_label = kwargs.pop("autoplak_receiver_label", "MicroLab")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return AutoplakSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    def encode_worklist(self, worklist, context):
        context_control_id = control_id = "MSG{}".format(
            context.get("control_id", int(time.time() * 1000))
        )
        if context["query"] and context["query"].control_id:
            context_control_id = context["query"].control_id
        message = self.create(
            sending_application=self.autoplak_sender_code,
            sending_facility=self.autoplak_sender_label,
            receiving_application=self.autoplak_receiver_code,
            receiving_facility=self.autoplak_receiver_label,
            date=context["now"],
            control_id=control_id,
        )
        seg_msa = message.acknowledgment.add(message_control_id=context_control_id)
        seg_qak = seg_msa.query_tag.add(order_control=context_control_id)
        seg_qpd = seg_qak.query_message.add(query_tag=context_control_id)
        sample = ""
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "S", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, worklist.ELSE: ""},
        ):
            for sample, action, priority in patient.samples:
                seq = 1
                seg_qpd.specimen_identification = sample.sample_id
                seg_spm = seg_qpd.specimen.add(
                    specimen_id=sample.sample_id,
                    specimen_type_code=sample.specimen,
                    specimen_type_text=sample.specimen,
                    collection_date=sample.collection_date,
                )
                seg_pid = seg_spm.patient.add(
                    first_name=patient.first_name,
                    last_name=patient.last_name,
                    birthdate=patient.birthdate,
                    sex=patient.sex,
                )
                for order in sample.orders:
                    placer_order_number = context.get(
                        "placer_order_number", int(time.time() * 1000 + seq)
                    )
                    seg_orc = seg_pid.infos.add(
                        placer_order_number=placer_order_number,
                        placer_identidier=placer_order_number,
                        date=sample.request_date,
                    )
                    seg_tq1 = seg_orc.sample_priority.add(priority=priority)
                    seg_tq1.orders.add(
                        placer_order_number=placer_order_number, code=order.code
                    )
        if sample:
            return message
        elif context["query"]:
            message = self.create(
                sending_application=self.autoplak_sender_code,
                sending_facility=self.autoplak_sender_label,
                receiving_application=self.autoplak_receiver_code,
                receiving_facility=self.autoplak_receiver_label,
                date=context["now"],
                control_id=control_id,
            )
            seg_msa = message.acknowledgment.add(message_control_id=control_id)
            seg_qak = seg_msa.query_tag.add(response_status="NF")
            for sample_id in [
                seg_q.specimen_identification for seg_q in context["query"].queries
            ]:
                seg_qak.query_message.add(
                    query_tag=control_id, specimen_identification=sample_id
                )
            return message

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_qpd in message.queries:
                sample = worklist.get_or_create(
                    type="patient", sample_id=seg_qpd.specimen_identification
                )
        else:
            worklist = Worklist(context="result")
            for seg_spm in message.specimen:
                sample = worklist.get_or_create(
                    sample_id=seg_spm.specimen_id, type="patient"
                )
                if seg_spm.patient:
                    specimen_container = seg_spm.patient[0].specimen_container
                else:
                    specimen_container = seg_spm.specimen_container
                for seg_sac in specimen_container:
                    if seg_sac.carrier_id and seg_sac.tray_id:
                        sample.rack = "{}-{}_{}-{}-{}".format(
                            seg_sac.carrier_id,
                            seg_sac.carrier_position,
                            seg_sac.tray_id,
                            seg_sac.tray_identifier,
                            seg_sac.tray_position,
                        )
                    boites = []
                    for seg_obr in seg_sac.orders:
                        boites.append(seg_obr.code)
                        for seg_orc in seg_obr.infos:
                            if seg_orc.date:
                                sample.results.add(
                                    code="{}DATETIME".format(
                                        seg_spm.specimen_type_code
                                    ),
                                    value=seg_orc.date.strftime("%Y/%m/%d %H:%M:%S"),
                                    date=message.date,
                                    operator=seg_sac.identifier,
                                    instrument=message.sending_application,
                                )
                        if seg_obr.infos:
                            results = seg_obr.infos[0].results
                        else:
                            results = seg_obr.results
                        for seg_obx in results:
                            # Id AUTOPLAK
                            sample.results.add(
                                code="{}ID".format(seg_spm.specimen_type_code),
                                value=seg_spm.autoplak_id,
                                date=message.date,
                                operator=seg_sac.identifier,
                                instrument=message.sending_application,
                            )
                            # Status du flacon
                            sample.results.add(
                                code="{}STATUS".format(seg_spm.specimen_type_code),
                                value=seg_obx.result_status,
                                date=message.date,
                                operator=seg_sac.identifier,
                                instrument=message.sending_application,
                            )
                    if boites:
                        sample.results.add(
                            code="{}_BOITES".format(seg_spm.specimen_type_code),
                            value=";".join(boites),
                            date=message.date,
                            operator=seg_sac.identifier,
                            instrument=message.sending_application,
                        )
        return worklist
