# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import MessageV2
from driverng.splitted import Date, Splitted, String


class QCMLTube(Splitted):
    SEPARATOR = "|"
    FIELDS = (
        String("type_line", default="1"),
        String("instrument_id"),
        String("tube"),
        Date("date", fmt_date="%d/%m/%Y %H:%M:%S"),
        String("fluide"),
    )


class QCMLResult(Splitted):
    SEPARATOR = "|"
    FIELDS = (
        String("type_line", default="2"),
        String("analyte_id"),
        String("result"),
        String("flags"),
        String("reactif_string"),
        String("calibration_string"),
        String("id_patient"),
        Date("birthdate", fmt_date="%d/%m/%Y"),
        String("sexe"),
        String("reactif_ref"),
    )


class QCMLPAT(list):
    def add(self, type_line, **kwargs):
        if type_line == 1:
            obj = QCMLTube(**kwargs)
        else:
            obj = QCMLResult(**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        for line in data.splitlines():
            if not line:
                continue
            self.append(QCMLResult().parse(line))
        return self

    def to_string(self):
        return "\n".join([e.to_string() for e in self])


class Driver(MessageV2):
    SPEC = {"code": "qcmlpat", "label": "QCML PATIENT"}
    DATE_FORMAT = "%d/%m/%Y %H:%M:%S"

    def create(self, **kwargs):
        return QCMLPAT(**kwargs)

    def parse(self, data):
        return QCMLPAT().parse(data)

    def to_string(self, message):
        data = message.to_string()
        return data

    def encode_worklist(self, worklist, context):
        # Uniquement les contextes de type result
        if worklist.context != "order":
            return None
        msg = self.create()
        # Uniquement les samples de type patient
        for sample in worklist.samples:
            if sample.type != "patient":
                continue
            if not sample.results:
                continue

            msg.add(
                1,
                instrument_id=sample.results[0].instrument,
                tube=sample.sample_id,
                date=sample.collection_date,
                fluide=sample.specimen,
            )
            for result in sample.results:
                reagent = ""
                calibration_string = ""
                reactif_string = ""
                for lot in result.lots:
                    if lot.type == "reagent":
                        reagent = lot.lot
                        reactif_string = lot.name or lot.lot
                        if lot.expiration:
                            reactif_string += "^%s" % lot.expiration.strftime(
                                self.DATE_FORMAT
                            )
                    if lot.type == "calibration":
                        calibration_string = lot.name or lot.lot
                        if lot.expiration:
                            calibration_string += "^%s" % lot.expiration.strftime(
                                self.DATE_FORMAT
                            )
                msg.add(
                    2,
                    analyte_id=result.code,
                    result=result.value,
                    fluide=sample.specimen,
                    flags="\\".join(result.flags),
                    reactif_string=reactif_string,
                    calibration_string=calibration_string,
                    id_patient=sample.lab_patient_id,
                    sexe=sample.sex,
                    birthdate=sample.birthdate,
                    reactif_ref=reagent,
                )
        if msg:
            return msg
        else:
            return None
