# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging

import six
from six import BytesIO, StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import String, Multiple

logger = logging.getLogger("driverng.slan_csv")


class SLANCsvResult(object):
    separator = ";"
    FIELDS = [
        String("identcol"),
        Multiple(name="ContentCols", fields=[String("content")], separator=separator),
    ]

    def __init__(self, **kwargs):
        self.separator = kwargs.pop("separator")
        super(SLANCsvResult, self).__init__(**kwargs)

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class SLANCsvResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: SLANCsvResult}

    def __init__(self, **kwargs):
        self.separator = kwargs.pop("separator")
        super(SLANCsvResultsMultiple, self).__init__(**kwargs)

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](separator=self.separator, **kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(self.separator))
        self.date_resu = None
        self.instrument_serial_number = None
        for row in reader:
            if six.PY2:
                line = [
                    c.decode("utf-8") for c in row
                ]  # PY2 csv reader est imcompatible avec unicode
            else:
                line = row
            if not line:
                continue
            obj = self.add(type_message=MessageV2.RESULT)
            obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "slan_csv", "label": "SANSURE SLAN 96 (csv)"}
    OPTIONS = [
        MessageV2.Option(
            "separator", label="séparateur csv (';' par défaut)", default=";"
        )
    ]

    def __init__(self, **kwargs):
        self.separator = kwargs.pop("separator", ";")
        super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = SLANCsvResultsMultiple(separator=self.separator)
        obj.parse(data)
        return obj

    WELL = "Well"
    PROJECT = "Project"
    SAMPLE_NAME = "Sample Name"
    CHANNEL = "Channel"
    DYE = "Dye"
    TARGET = "Target"
    CT = "Ct"

    IGNORE_LIST = [
        WELL,
        SAMPLE_NAME,
        "Tube No.",
        "Tube Name",
        CHANNEL,
        "Patient Name",
        "Sex",
        "Age",
        "Doctor",
        "Department",
        "Diagnosis",
        "Case ID",
        "Bed ID",
        "In-patient ID",
        "Out-patient ID",
        "Remark",
    ]

    EXPERIMENT_NAME = "Experiment Name"
    START_EXPERIMENT = "Start at"
    OPERATOR = "Operator"
    INSTRUMENT = "[Instrument]"
    INSTRUMENT_MODEL = "Model"
    INSTRUMENT_SN = "S/N"

    AMPLIFICATION_CURVE = "Amplification Curve"
    RAW_AMPLIFICATION_CURVE = "Raw Amplification Curve"

    def get_global_info(self, message):

        ginfo = {}
        get_instrument = False
        instrument_model = ""
        instrument_sn = ""
        wellcol = -1
        projectcol = -1
        sampleidcol = -1
        channelcol = -1
        dyecol = -1
        targetcol = -1
        ctcol = -1

        for m in message:
            if m.identcol == self.EXPERIMENT_NAME:
                ginfo[self.EXPERIMENT_NAME] = "{}".format(m.ContentCols[0].strip())
            elif m.identcol == self.START_EXPERIMENT:
                ginfo[self.START_EXPERIMENT] = datetime.datetime.strptime(
                    m.ContentCols[0].strip(), "%Y-%m-%d %H:%M:%S"
                )
            elif m.identcol == self.OPERATOR:
                ginfo[self.OPERATOR] = "{}".format(m.ContentCols[0].strip())
            elif m.identcol == self.WELL:
                wellcol = 0
                for index, colContent in enumerate(m.ContentCols):
                    colContent = colContent.strip()
                    if colContent == self.PROJECT:
                        projectcol = index
                    elif colContent == self.SAMPLE_NAME:
                        sampleidcol = index
                    elif colContent == self.CHANNEL:
                        channelcol = index
                    elif colContent == self.DYE:
                        dyecol = index
                    elif colContent == self.TARGET:
                        targetcol = index
                    elif colContent == self.CT:
                        ctcol = index
            elif m.identcol == self.INSTRUMENT:
                get_instrument = True
            elif get_instrument:
                if m.identcol == self.INSTRUMENT_MODEL:
                    instrument_model = m.ContentCols[0]
                elif m.identcol == self.INSTRUMENT_SN:
                    instrument_sn = m.ContentCols[0]

        if instrument_sn:
            ginfo[self.INSTRUMENT] = instrument_sn
        elif instrument_model:
            ginfo[self.INSTRUMENT] = instrument_model
        else:
            ginfo[self.INSTRUMENT] = ""

        if (
            wellcol == -1
            or projectcol == -1
            or sampleidcol == -1
            or channelcol == -1
            or dyecol == -1
            or targetcol == -1
            or ctcol == -1
        ):
            cols = {
                "wellcol": wellcol if wellcol != -1 else "missing",
                "projectcol": projectcol if projectcol != -1 else "missing",
                "sampleidcol": sampleidcol if sampleidcol != -1 else "missing",
                "channelcol": channelcol if channelcol != -1 else "missing",
                "dyecol": dyecol if dyecol != -1 else "missing",
                "targetcol": targetcol if targetcol != -1 else "missing",
                "ctcol": ctcol if ctcol != -1 else "missing",
            }
            logger.warning("Il manque des colonnes obligatoires %s", cols)
            return None

        ginfo["wellcol"] = wellcol
        ginfo["projectcol"] = projectcol
        ginfo["sampleidcol"] = sampleidcol
        ginfo["channelcol"] = channelcol
        ginfo["dyecol"] = dyecol
        ginfo["targetcol"] = targetcol
        return ginfo

    def get_result(self, code, value):
        return {
            "code": code,
            "value": value,
            "instrument": self.ginfo[self.INSTRUMENT],
            "date": self.ginfo[self.START_EXPERIMENT],
            "operator": self.ginfo[self.OPERATOR],
        }

    def get_result_graph(self, graph_type, graph):
        return {
            "code": graph_type,
            "value": "GRAPHE",
            "date": self.ginfo[self.START_EXPERIMENT],
            "graph": {"graph_type": "area", "points": graph},
            "instrument": self.ginfo[self.INSTRUMENT],
            "operator": self.ginfo[self.OPERATOR],
        }

    def get_sample_id(self, ligne):
        sample_get = ligne.ContentCols[self.ginfo["sampleidcol"]].strip()
        if sample_get in ("+", "-", "/", ""):
            return ""
        return sample_get

    def decode_message(self, message):
        logger.info("decode_message")
        worklist = Worklist(context="result")
        ginfo = self.get_global_info(message)
        if not ginfo:
            return worklist

        logger.info("decode_message ok fichier csv")
        self.ginfo = ginfo

        CONTENT = ()
        for m in message:
            if m.identcol == self.WELL:
                CONTENT = m.ContentCols
                ROWS_COUNT = len(m.ContentCols)
            elif CONTENT and ROWS_COUNT == len(m.ContentCols):

                well = m.identcol
                project = m.ContentCols[ginfo["projectcol"]]
                sample_id = self.get_sample_id(m)
                channel = m.ContentCols[ginfo["channelcol"]]
                dye = m.ContentCols[ginfo["dyecol"]]
                target = m.ContentCols[ginfo["targetcol"]]

                if (
                    len(well.strip()) == 0
                    or len(project.strip()) == 0
                    or len(sample_id.strip()) == 0
                    or len(channel.strip()) == 0
                    or len(dye.strip()) == 0
                    or len(target.strip()) == 0
                ):
                    continue

                rack = "{}".format(well)
                sample = worklist.get_or_create(
                    sample_id=sample_id, rack=rack, type="patient"
                )

                graph = None
                get_graph_point = False
                raw_graph = None
                get_raw_graph_point = False
                results = sample.results
                for index, colContent in enumerate(m.ContentCols):
                    if CONTENT[index] in self.IGNORE_LIST:
                        continue

                    if CONTENT[index] == self.AMPLIFICATION_CURVE:
                        graph_type = "{}_D{}_GRAPHE".format(project, dye)
                        graph = []
                        get_graph_point = True
                        get_raw_graph_point = False
                    elif CONTENT[index] == self.RAW_AMPLIFICATION_CURVE:
                        raw_graph_type = "{}_D{}_GRAPHE-RAW".format(project, dye)
                        raw_graph = []
                        get_raw_graph_point = True
                        get_graph_point = False
                    elif get_graph_point:
                        if len(colContent.strip()):
                            graph.append(float(colContent.replace(",", ".")))
                        else:
                            get_graph_point = False
                    elif get_raw_graph_point:
                        if len(colContent.strip()):
                            raw_graph.append(float(colContent.replace(",", ".")))
                        else:
                            get_raw_graph_point = False

                    else:
                        code = "{}_C{}_{}".format(project, channel, CONTENT[index])
                        code_dye = "{}_D{}_{}".format(project, dye, CONTENT[index])
                        code_target = "{}_T{}_{}".format(
                            project, target, CONTENT[index]
                        )
                        value = "{}".format(colContent.replace(",", "."))
                        results.add(self.get_result(code, value))
                        results.add(self.get_result(code_dye, value))
                        results.add(self.get_result(code_target, value))

                if graph:
                    results.add(self.get_result_graph(graph_type, graph))
                if raw_graph:
                    results.add(self.get_result_graph(raw_graph_type, raw_graph))

        return worklist
